@@ -52,15 +52,9 @@ def addExecutionSessionToSampleCache(executionSessionId, config):
 
     sampleCacheDir = config.getKwolaUserDataDirectory("prepared_samples")
 
-    if config['enable_reward_normalization']:
-        with open(os.path.join(config.getKwolaUserDataDirectory("models"), "reward_normalizer"), "rb") as normalizerFile:
-            trainingRewardNormalizer = pickle.load(normalizerFile)
-    else:
-        trainingRewardNormalizer = None
-
     executionSession = ExecutionSession.loadFromDisk(executionSessionId, config)
 
-    batches = agent.prepareBatchesForExecutionSession(executionSession, trainingRewardNormalizer=trainingRewardNormalizer)
+    batches = agent.prepareBatchesForExecutionSession(executionSession)
 
     for traceIndex, traceBatch in zip(range(len(executionSession.executionTraces) - 1), batches):
         traceId = traceBatch['traceIds'][0]
@@ -241,7 +235,7 @@ def updateTraceRewardLoss(traceId, sampleRewardLoss, configDir):
     trace.saveToDisk(config)
 
 
-def prepareAndLoadBatchesSubprocess(configDir, batchDirectory, subProcessCommandQueue, subProcessBatchResultQueue, createRewardNormalizer=True, subprocessIndex=0):
+def prepareAndLoadBatchesSubprocess(configDir, batchDirectory, subProcessCommandQueue, subProcessBatchResultQueue, subprocessIndex=0):
     try:
         config = Configuration(configDir)
 
@@ -284,26 +278,6 @@ def prepareAndLoadBatchesSubprocess(configDir, batchDirectory, subProcessCommand
 
         print(datetime.now(), f"[{os.getpid()}]", f"Finished loading of weight datas for {len(executionTraceWeightDatas)} execution traces.", flush=True)
 
-        if config['enable_reward_normalization']:
-            rewardNormalizerPath = os.path.join(config.getKwolaUserDataDirectory("models"), "reward_normalizer")
-            if createRewardNormalizer:
-                print(datetime.now(), f"[{os.getpid()}]", "Starting creation of the reward normalizer.", flush=True)
-
-                trainingRewardNormalizer = DeepLearningAgent.createTrainingRewardNormalizer(random.sample(executionSessionIds, min(len(executionSessionIds), config['training_reward_normalizer_fit_population_size'])), configDir)
-
-                with open(rewardNormalizerPath, "wb") as normalizerFile:
-                    pickle.dump(trainingRewardNormalizer, normalizerFile)
-
-                del trainingRewardNormalizer
-                print(datetime.now(), f"[{os.getpid()}]", "Finished creation of the reward normalizer.", flush=True)
-            else:
-                # Wait for a reward normalizer to be created by another of the batch-prep sub-proccesses
-                while True:
-                    if os.path.exists(rewardNormalizerPath):
-                        break
-                    else:
-                        time.sleep(1)
-
         del testingSteps, executionSessionIds, executionSessionFutures, executionSessions, executionTraceFutures
         print(datetime.now(), f"[{os.getpid()}]", "Finished initialization for batch preparation sub process.", flush=True)
 
@@ -353,7 +327,7 @@ def prepareAndLoadBatchesSubprocess(configDir, batchDirectory, subProcessCommand
 
                         # We do this check here because saving execution traces is actually a pretty CPU heavy process,
                         # so we only want to do it if the loss has actually changed by a significant degree
-                        differenceRatio = abs(traceWeightData['weight'] - sampleRewardLoss) / traceWeightData['weight']
+                        differenceRatio = abs(traceWeightData['weight'] - sampleRewardLoss) / (traceWeightData['weight'] + 1e-6)
                         if differenceRatio > config['training_trace_selection_min_loss_ratio_difference_for_save']:
                             traceWeightData['weight'] = sampleRewardLoss
                             if executionTraceId not in executionTraceSaveFutures or executionTraceSaveFutures[executionTraceId].ready():
@@ -499,9 +473,9 @@ def loadExecutionTraceWeightData(traceId, sessionId, configDir):
 def saveExecutionTraceWeightData(traceWeightData, configDir):
     config = Configuration(configDir)
 
-    weightFile = os.path.join(config.getKwolaUserDataDirectory("execution_trace_weight_files"), traceId + ".json")
+    weightFile = os.path.join(config.getKwolaUserDataDirectory("execution_trace_weight_files"), traceWeightData['id'] + ".json")
 
-    with open(weightFile, "rw") as f:
+    with open(weightFile, "wt") as f:
         json.dump(f, traceWeightData)
 
 
@@ -511,7 +485,7 @@ def runTrainingStep(configDir, trainingSequenceId, trainingStepIndex, gpu=None):
     if gpu is not None:
         for subprocessIndex in range(10):
             try:
-                torch.distributed.init_process_group(backend="gloo", world_size=2, rank=gpu, init_method="file:///tmp/kwola_distributed_coordinator", )
+                torch.distributed.init_process_group(backend="gloo", world_size=torch.cuda.device_count(), rank=gpu, init_method="file:///tmp/kwola_distributed_coordinator", )
                 break
             except RuntimeError:
                 time.sleep(1)
@@ -570,9 +544,7 @@ def runTrainingStep(configDir, trainingSequenceId, trainingStepIndex, gpu=None):
             subProcessCommandQueue = multiprocessing.Queue()
             subProcessBatchResultQueue = multiprocessing.Queue()
 
-            createRewardNormalizer = config['enable_reward_normalization'] and bool(subprocessIndex == 0 and (gpu == 0 or gpu is None))
-
-            subProcess = multiprocessing.Process(target=prepareAndLoadBatchesSubprocess, args=(configDir, batchDirectory, subProcessCommandQueue, subProcessBatchResultQueue, createRewardNormalizer, subprocessIndex))
+            subProcess = multiprocessing.Process(target=prepareAndLoadBatchesSubprocess, args=(configDir, batchDirectory, subProcessCommandQueue, subProcessBatchResultQueue, subprocessIndex))
             subProcess.start()
             atexit.register(lambda: subProcess.terminate())
 
